@@ -1,3 +1,19 @@
+/*
+ * Copyright 2024 Vsevolod Stakhov
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 /* Copyright (c) 2014, Vsevolod Stakhov
  * All rights reserved.
  *
@@ -28,6 +44,7 @@
 #include "ucl.h"
 #include "ucl_internal.h"
 #include "lua_ucl.h"
+#include "kvec.h"
 #include <strings.h>
 
 /***
@@ -69,6 +86,7 @@ func = "huh";
 #define PARSER_META "ucl.parser.meta"
 #define EMITTER_META "ucl.emitter.meta"
 #define NULL_META "ucl.null.meta"
+#define ITER_META "ucl.object.iter"
 #define OBJECT_META "ucl.object.meta"
 #define UCL_OBJECT_TYPE_META "ucl.type.object"
 #define UCL_ARRAY_TYPE_META "ucl.type.array"
@@ -79,10 +97,20 @@ static int ucl_object_lua_push_scalar(lua_State *L, const ucl_object_t *obj, int
 static int ucl_object_push_lua_common(lua_State *L, const ucl_object_t *obj, int flags);
 static ucl_object_t *ucl_object_lua_fromtable(lua_State *L, int idx, ucl_string_flags_t flags);
 static ucl_object_t *ucl_object_lua_fromelt(lua_State *L, int idx, ucl_string_flags_t flags);
+static ucl_object_t *lua_ucl_object_get(lua_State *L, int index);
+static int lua_ucl_type(lua_State *L);
+static int lua_ucl_pairs(lua_State *L);
+static int lua_ucl_ipairs(lua_State *L);
+static int lua_ucl_len(lua_State *L);
+static int lua_ucl_index(lua_State *L);
+static int lua_ucl_newindex(lua_State *L);
+static int lua_ucl_object_tostring(lua_State *L);
+static int lua_ucl_object_unwrap(lua_State *L);
+static int lua_ucl_object_validate(lua_State *L);
 
 static void *ucl_null;
 
-struct _rspamd_lua_text {
+struct rspamd_compat_lua_text {
 	const char *start;
 	unsigned int len;
 	unsigned int flags;
@@ -519,7 +547,7 @@ ucl_object_lua_fromelt(lua_State *L, int idx, ucl_string_flags_t flags)
 		}
 		else {
 			/* Assume it is a text like object */
-			struct _rspamd_lua_text *t = lua_touserdata(L, idx);
+			struct rspamd_compat_lua_text *t = lua_touserdata(L, idx);
 
 			if (t) {
 				if (t->len > 0) {
@@ -551,7 +579,22 @@ ucl_object_lua_fromelt(lua_State *L, int idx, ucl_string_flags_t flags)
 		}
 		else {
 			if (type == LUA_TTABLE) {
-				obj = ucl_object_lua_fromtable(L, idx, flags);
+				lua_rawgeti(L, idx, 0);
+
+				if (lua_type(L, -1) == LUA_TUSERDATA) {
+					/* It is a cloaked ucl object */
+					obj = lua_ucl_object_get(L, idx);
+
+					if (obj) {
+						obj = ucl_object_ref(obj);
+					}
+				}
+
+				lua_pop(L, 1);
+
+				if (obj == NULL) {
+					obj = ucl_object_lua_fromtable(L, idx, flags);
+				}
 			}
 			else if (type == LUA_TFUNCTION) {
 				fd = malloc(sizeof(*fd));
@@ -679,16 +722,54 @@ lua_ucl_parser_get(lua_State *L, int index)
 static ucl_object_t *
 lua_ucl_object_get(lua_State *L, int index)
 {
-	return *((ucl_object_t **) luaL_checkudata(L, index, OBJECT_META));
+	if (lua_istable(L, index)) {
+		ucl_object_t *res = NULL;
+		lua_rawgeti(L, index, 0);
+
+		if (lua_isuserdata(L, -1)) {
+			res = *((ucl_object_t **) lua_touserdata(L, -1));
+		}
+
+		lua_pop(L, 1);
+
+		return res;
+	}
+
+	return NULL;
 }
 
-static void
-lua_ucl_push_opaque(lua_State *L, ucl_object_t *obj)
+void ucl_object_push_lua_unwrapped(lua_State *L, const ucl_object_t *obj)
 {
 	ucl_object_t **pobj;
 
+	/* We create a plain lua table with the following elements:
+	 * [0] - ucl object as userdata
+	 * ["method"] - methods
+	 */
+	lua_createtable(L, 1, 9);
 	pobj = lua_newuserdata(L, sizeof(*pobj));
-	*pobj = obj;
+	*pobj = ucl_object_ref(obj);
+	lua_rawseti(L, -2, 0);
+
+	lua_pushcfunction(L, lua_ucl_index);
+	lua_setfield(L, -2, "at");
+	lua_pushcfunction(L, lua_ucl_type);
+	lua_setfield(L, -2, "type");
+	lua_pushcfunction(L, lua_ucl_pairs);
+	lua_setfield(L, -2, "pairs");
+	lua_pushcfunction(L, lua_ucl_ipairs);
+	lua_setfield(L, -2, "ipairs");
+	lua_pushcfunction(L, lua_ucl_len);
+	lua_setfield(L, -2, "len");
+	lua_pushcfunction(L, lua_ucl_object_tostring);
+	lua_setfield(L, -2, "tostring");
+	lua_pushcfunction(L, lua_ucl_object_unwrap);
+	lua_setfield(L, -2, "unwrap");
+	lua_pushcfunction(L, lua_ucl_object_unwrap);
+	lua_setfield(L, -2, "tolua");
+	lua_pushcfunction(L, lua_ucl_object_validate);
+	lua_setfield(L, -2, "validate");
+
 	luaL_getmetatable(L, OBJECT_META);
 	lua_setmetatable(L, -2);
 }
@@ -878,7 +959,7 @@ static int
 lua_ucl_parser_parse_text(lua_State *L)
 {
 	struct ucl_parser *parser;
-	struct _rspamd_lua_text *t;
+	struct rspamd_compat_lua_text *t;
 	enum ucl_parse_type type = UCL_PARSE_UCL;
 	int ret = 2;
 
@@ -890,7 +971,7 @@ lua_ucl_parser_parse_text(lua_State *L)
 	else if (lua_type(L, 2) == LUA_TSTRING) {
 		const char *s;
 		gsize len;
-		static struct _rspamd_lua_text st_t;
+		static struct rspamd_compat_lua_text st_t;
 
 		s = lua_tolstring(L, 2, &len);
 		st_t.start = s;
@@ -969,7 +1050,8 @@ lua_ucl_parser_get_object_wrapped(lua_State *L)
 	obj = ucl_parser_get_object(parser);
 
 	if (obj != NULL) {
-		lua_ucl_push_opaque(L, obj);
+		ucl_object_push_lua_unwrapped(L, obj);
+		ucl_object_unref(obj);
 	}
 	else {
 		lua_pushnil(L);
@@ -1128,8 +1210,9 @@ lua_ucl_object_tostring(lua_State *L)
 	enum ucl_emitter format = UCL_EMIT_JSON_COMPACT;
 
 	obj = lua_ucl_object_get(L, 1);
+	int type = ucl_object_type(obj);
 
-	if (obj) {
+	if (type == UCL_ARRAY || type == UCL_OBJECT) {
 		if (lua_gettop(L) > 1) {
 			if (lua_type(L, 2) == LUA_TSTRING) {
 				const char *strtype = lua_tostring(L, 2);
@@ -1140,9 +1223,12 @@ lua_ucl_object_tostring(lua_State *L)
 
 		return lua_ucl_to_string(L, obj, format);
 	}
-	else {
+	else if (type == UCL_NULL) {
 		lua_pushnil(L);
 	}
+	else {
+		ucl_object_lua_push_scalar(L, obj, 0);
+	}
 
 	return 1;
 }
@@ -1211,7 +1297,8 @@ lua_ucl_object_validate(lua_State *L)
 				lua_pushnil(L);
 
 				if (ext_refs) {
-					lua_ucl_push_opaque(L, ext_refs);
+					ucl_object_push_lua_unwrapped(L, ext_refs);
+					ucl_object_unref(ext_refs);
 				}
 			}
 			else {
@@ -1219,7 +1306,8 @@ lua_ucl_object_validate(lua_State *L)
 				lua_pushfstring(L, "validation error: %s", err.msg);
 
 				if (ext_refs) {
-					lua_ucl_push_opaque(L, ext_refs);
+					ucl_object_push_lua_unwrapped(L, ext_refs);
+					ucl_object_unref(ext_refs);
 				}
 			}
 		}
@@ -1229,7 +1317,8 @@ lua_ucl_object_validate(lua_State *L)
 			lua_pushfstring(L, "cannot find the requested path: %s", path);
 
 			if (ext_refs) {
-				lua_ucl_push_opaque(L, ext_refs);
+				ucl_object_push_lua_unwrapped(L, ext_refs);
+				ucl_object_unref(ext_refs);
 			}
 		}
 	}
@@ -1257,6 +1346,338 @@ lua_ucl_object_gc(lua_State *L)
 	return 0;
 }
 
+static int
+lua_ucl_iter_gc(lua_State *L)
+{
+	ucl_object_iter_t it;
+
+	it = *((ucl_object_iter_t *) lua_touserdata(L, 1));
+
+	if (it) {
+		ucl_object_iterate_free(it);
+	}
+
+	return 0;
+}
+
+static int
+lua_ucl_index(lua_State *L)
+{
+	ucl_object_t *obj;
+
+	obj = lua_ucl_object_get(L, 1);
+
+	if (lua_type(L, 2) == LUA_TSTRING) {
+		/* Index by string */
+
+		if (ucl_object_type(obj) == UCL_OBJECT) {
+			size_t len;
+			const char *key = lua_tolstring(L, 2, &len);
+			const ucl_object_t *elt;
+
+			elt = ucl_object_lookup_len(obj, key, strlen(key));
+
+			if (elt) {
+				ucl_object_push_lua_unwrapped(L, elt);
+			}
+			else {
+				lua_pushnil(L);
+			}
+
+			return 1;
+		}
+		else {
+			return luaL_error(L, "cannot index non-object: %s", ucl_object_type_to_string(ucl_object_type(obj)));
+		}
+	}
+	else if (lua_type(L, 2) == LUA_TNUMBER) {
+		/* Index by number */
+		if (ucl_object_type(obj) == UCL_ARRAY) {
+			/* +1 as Lua indexes elements from 1 and ucl indexes them from 0 */
+			lua_Integer idx = lua_tointeger(L, 2) + 1;
+			const ucl_object_t *elt;
+
+			elt = ucl_array_find_index(obj, idx);
+
+			if (elt) {
+				ucl_object_push_lua_unwrapped(L, elt);
+			}
+			else {
+				lua_pushnil(L);
+			}
+
+			return 1;
+		}
+		else {
+			return luaL_error(L, "cannot index non-array: %s", ucl_object_type_to_string(ucl_object_type(obj)));
+		}
+	}
+	else {
+		return luaL_error(L, "invalid index type: %s", lua_typename(L, lua_type(L, 2)));
+	}
+}
+
+static int
+lua_ucl_newindex(lua_State *L)
+{
+	ucl_object_t *obj;
+	obj = lua_ucl_object_get(L, 1);
+	int key_type = lua_type(L, 2);
+
+	if (ucl_object_type(obj) == UCL_OBJECT) {
+		if (key_type == LUA_TSTRING) {
+			size_t keylen;
+			const char *key = lua_tolstring(L, 2, &keylen);
+
+			ucl_object_t *value_obj = lua_ucl_object_get(L, 3);
+			if (value_obj) {
+				value_obj = ucl_object_ref(value_obj);
+			}
+			else {
+				value_obj = ucl_object_lua_import(L, 3);
+			}
+
+			if (ucl_object_lookup_len(obj, key, keylen) != NULL) {
+				if (value_obj != NULL) {
+					ucl_object_replace_key(obj, value_obj, key, keylen, true);
+				}
+				else {
+					/* Delete key */
+					ucl_object_delete_keyl(obj, key, keylen);
+				}
+			}
+			else {
+				if (value_obj != NULL) {
+					ucl_object_insert_key(obj, value_obj, key, keylen, true);
+				}
+				/* Do nothing if value_obj is null, like Lua does */
+			}
+		}
+		else if (key_type == LUA_TNUMBER) {
+			/*
+			 * We have some object but wanted it to be an array
+			 * In Lua, this is allowed but UCL has distinction between array and object
+			 * Here, we allow one thing: if an object is empty, and we want a numeric index, we convert it to an array
+			 */
+			lua_Integer idx = lua_tointeger(L, 2);
+			if (idx == 1 && obj->len == 0 && obj->value.ov == NULL) {
+				/* UCL preallocate arrays, but it is not a requirement once av is NULL */
+				obj->type = UCL_ARRAY;
+				ucl_object_t *value_obj = lua_ucl_object_get(L, 3);
+				if (value_obj) {
+					value_obj = ucl_object_ref(value_obj);
+				}
+				else {
+					value_obj = ucl_object_lua_import(L, 3);
+				}
+
+				if (value_obj == NULL) {
+					return luaL_error(L, "invalid value type: %s", lua_typename(L, lua_type(L, 3)));
+				}
+				ucl_array_append(obj, value_obj);
+			}
+			else {
+				return luaL_error(L, "invalid index type for an object: %s", lua_typename(L, key_type));
+			}
+		}
+		else {
+			return luaL_error(L, "invalid index type for an object: %s", lua_typename(L, key_type));
+		}
+
+		return 0;
+	}
+	else if (ucl_object_type(obj) == UCL_ARRAY) {
+		if (key_type == LUA_TNUMBER) {
+			lua_Integer idx = lua_tointeger(L, 2);
+
+			ucl_object_t *value_obj = lua_ucl_object_get(L, 3);
+			if (value_obj) {
+				value_obj = ucl_object_ref(value_obj);
+			}
+			else {
+				value_obj = ucl_object_lua_import(L, 3);
+			}
+
+			if (value_obj == NULL) {
+				return luaL_error(L, "invalid value type: %s", lua_typename(L, lua_type(L, 3)));
+			}
+
+			/* Lua allows sparse arrays and ucl does not
+			 * So we have 2 options:
+			 * 1) Idx is some existing index, so we need to replace it
+			 * 3) Idx is #len, so we append it
+			 * Everything else is an error
+			 */
+			if (idx == ucl_array_size(obj) + 1) {
+				ucl_array_append(obj, value_obj);
+			}
+			else if (idx >= 1 && idx <= ucl_array_size(obj)) {
+				ucl_array_replace_index(obj, value_obj, idx - 1);
+			}
+			else {
+				ucl_object_unref(value_obj);
+
+				return luaL_error(L, "invalid index for array: %d", (int) idx);
+			}
+		}
+		else if (key_type == LUA_TSTRING) {
+			/*
+			 * We have some array but wanted it to be an object
+			 * In Lua, this is allowed but UCL has distinction between array and object
+			 * Here, we allow one thing: if an array is empty, and we want a string index, we convert it to an object
+			 * The biggest issue is that ucl array is preallocated in general, so we need to free it somehow
+			 */
+			/*
+			 * Dirty hacks are here
+			 */
+			kvec_t(ucl_object_t *) *real_ar = obj->value.av;
+
+			if (real_ar) {
+				kv_destroy(*real_ar);
+			}
+			UCL_FREE(sizeof(*real_ar), real_ar);
+			obj->value.av = NULL;
+			obj->type = UCL_OBJECT;
+
+			size_t keylen;
+			const char *key = lua_tolstring(L, 2, &keylen);
+
+			ucl_object_t *value_obj = lua_ucl_object_get(L, 3);
+			if (value_obj) {
+				value_obj = ucl_object_ref(value_obj);
+			}
+			else {
+				value_obj = ucl_object_lua_import(L, 3);
+			}
+
+			if (value_obj) {
+				ucl_object_insert_key(obj, value_obj, key, keylen, true);
+			}
+			else {
+				return luaL_error(L, "invalid value type: %s", lua_typename(L, lua_type(L, 3)));
+			}
+		}
+		else {
+			return luaL_error(L, "invalid index type for an array: %s", lua_typename(L, key_type));
+		}
+
+		return 0;
+	}
+	else {
+		return luaL_error(L, "invalid index type: %s (obj type: %s)", lua_typename(L, key_type),
+						  ucl_object_type_to_string(ucl_object_type(obj)));
+	}
+}
+
+static int
+lua_ucl_type(lua_State *L)
+{
+	ucl_object_t *obj;
+
+	obj = lua_ucl_object_get(L, 1);
+	lua_pushstring(L, ucl_object_type_to_string(ucl_object_type(obj)));
+
+	return 1;
+}
+
+static int
+lua_ucl_object_iter(lua_State *L)
+{
+	ucl_object_iter_t it;
+	const ucl_object_t *cur;
+
+	it = *((ucl_object_iter_t *) lua_touserdata(L, 1));
+	cur = ucl_object_iterate_safe(it, true);
+
+	if (cur) {
+		if (ucl_object_key(cur)) {
+			size_t klen;
+			const char *k = ucl_object_keyl(cur, &klen);
+			lua_pushlstring(L, k, klen);
+		}
+		else {
+			if (lua_type(L, 2) == LUA_TNUMBER) {
+				lua_Integer idx = lua_tointeger(L, 2);
+				if (idx >= 0) {
+					lua_pushinteger(L, idx + 1);
+				}
+			}
+			else {
+				lua_pushnumber(L, -1);
+			}
+		}
+		ucl_object_push_lua_unwrapped(L, cur);
+
+		return 2;
+	}
+	else {
+		lua_pushnil(L);
+
+		return 1;
+	}
+}
+
+static int
+lua_ucl_pairs(lua_State *L)
+{
+	ucl_object_t *obj;
+
+	obj = lua_ucl_object_get(L, 1);
+	int t = ucl_object_type(obj);
+
+	if ((obj) && (t == UCL_ARRAY || t == UCL_OBJECT || obj->next != NULL)) {
+		/* iter_func, ucl_object_t, iter */
+		lua_pushcfunction(L, lua_ucl_object_iter);
+		ucl_object_iter_t *pit = lua_newuserdata(L, sizeof(ucl_object_iter_t *));
+		luaL_getmetatable(L, ITER_META);
+		lua_setmetatable(L, -2);
+		ucl_object_iter_t it = ucl_object_iterate_new(obj);
+		*pit = it;
+		lua_pushnumber(L, -1);
+
+		return 3;
+	}
+	else {
+		return luaL_error(L, "invalid object type for pairs: %s", ucl_object_type_to_string(t));
+	}
+}
+
+static int
+lua_ucl_len(lua_State *L)
+{
+	ucl_object_t *obj;
+
+	obj = lua_ucl_object_get(L, 1);
+	lua_pushinteger(L, obj->len);
+
+	return 1;
+}
+
+static int
+lua_ucl_ipairs(lua_State *L)
+{
+	ucl_object_t *obj;
+
+	obj = lua_ucl_object_get(L, 1);
+	int t = ucl_object_type(obj);
+
+	if ((obj) && (t == UCL_ARRAY || obj->next != NULL)) {
+		/* iter_func, ucl_object_t, iter */
+		lua_pushcfunction(L, lua_ucl_object_iter);
+		ucl_object_iter_t *pit = lua_newuserdata(L, sizeof(ucl_object_iter_t *));
+		luaL_getmetatable(L, ITER_META);
+		lua_setmetatable(L, -2);
+		ucl_object_iter_t it = ucl_object_iterate_new(obj);
+		*pit = it;
+		lua_pushnumber(L, 0);
+
+		return 3;
+	}
+	else {
+		return luaL_error(L, "invalid object type for ipairs: %s", ucl_object_type_to_string(t));
+	}
+}
+
 static void
 lua_ucl_parser_mt(lua_State *L)
 {
@@ -1300,31 +1721,43 @@ lua_ucl_object_mt(lua_State *L)
 {
 	luaL_newmetatable(L, OBJECT_META);
 
-	lua_pushvalue(L, -1);
+	lua_pushcfunction(L, lua_ucl_index);
 	lua_setfield(L, -2, "__index");
+	lua_pushcfunction(L, lua_ucl_newindex);
+	lua_setfield(L, -2, "__newindex");
+	/* Usable merely with lua 5.2+ */
+	lua_pushcfunction(L, lua_ucl_ipairs);
+	lua_setfield(L, -2, "__ipairs");
+	/* Usable merely with lua 5.2+ */
+	lua_pushcfunction(L, lua_ucl_pairs);
+	lua_setfield(L, -2, "__pairs");
+
+	/* Access UCL elements using `:at` method */
+	lua_pushcfunction(L, lua_ucl_index);
+	lua_setfield(L, -2, "at");
+
+	/* Usable merely with lua 5.2+ */
+	lua_pushcfunction(L, lua_ucl_len);
+	lua_setfield(L, -2, "__len");
 
 	lua_pushcfunction(L, lua_ucl_object_gc);
 	lua_setfield(L, -2, "__gc");
-
 	lua_pushcfunction(L, lua_ucl_object_tostring);
 	lua_setfield(L, -2, "__tostring");
 
-	lua_pushcfunction(L, lua_ucl_object_tostring);
-	lua_setfield(L, -2, "tostring");
-
-	lua_pushcfunction(L, lua_ucl_object_unwrap);
-	lua_setfield(L, -2, "unwrap");
-
-	lua_pushcfunction(L, lua_ucl_object_unwrap);
-	lua_setfield(L, -2, "tolua");
-
-	lua_pushcfunction(L, lua_ucl_object_validate);
-	lua_setfield(L, -2, "validate");
-
 	lua_pushstring(L, OBJECT_META);
 	lua_setfield(L, -2, "class");
 
 	lua_pop(L, 1);
+
+	luaL_newmetatable(L, ITER_META);
+	lua_pushcfunction(L, lua_ucl_iter_gc);
+	lua_setfield(L, -2, "__gc");
+
+	lua_pushstring(L, ITER_META);
+	lua_setfield(L, -2, "__tostring");
+
+	lua_pop(L, 1);
 }
 
 static void
