@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -42,12 +42,9 @@ local e = {
 -- %<number> means numeric arguments and %s means the next argument
 -- for example %1, %2, %s: %s would mean the third argument
 
-rspamd_logger.infox('a=%1, b=%2, c=%3, d=%4, e=%s', a, b, c, d, e)
+rspamd_logger.info('a=%1, b=%2, c=%3, d=%4, e=%s', a, b, c, d, e)
 -- Output: a=string, b=1.50000, c=1, d={[1] = aa, [2] = 1, [3] = bb} e={[key]=value, [key2]=1.0}
 
--- Legacy interface (can handle merely strings)
-rspamd_logger.info('Old stupid API')
-
 -- Create string using logger API
 local str = rspamd_logger.slog('a=%1, b=%2, c=%3, d=%4, e=%5', a, b, c, d, e)
 
@@ -180,13 +177,13 @@ static const struct luaL_reg loggerlib_f[] = {
 static void
 lua_common_log_line(GLogLevelFlags level,
 					lua_State *L,
-					const gchar *msg,
-					const gchar *uid,
-					const gchar *module,
-					gint stack_level)
+					const char *msg,
+					const char *uid,
+					const char *module,
+					int stack_level)
 {
 	lua_Debug d;
-	gchar func_buf[128], *p;
+	char func_buf[128], *p;
 
 	if (lua_getstack(L, stack_level, &d) == 1) {
 		(void) lua_getinfo(L, "Sl", &d);
@@ -226,54 +223,34 @@ lua_common_log_line(GLogLevelFlags level,
 }
 
 /*** Logger interface ***/
-static gint
+static int
 lua_logger_err(lua_State *L)
 {
-	LUA_TRACE_POINT;
-	const gchar *msg;
-	msg = luaL_checkstring(L, 1);
-	lua_common_log_line(G_LOG_LEVEL_CRITICAL, L, msg, NULL, NULL, 1);
-	return 0;
+	return lua_logger_errx(L);
 }
 
-static gint
+static int
 lua_logger_warn(lua_State *L)
 {
-	LUA_TRACE_POINT;
-	const gchar *msg;
-	msg = luaL_checkstring(L, 1);
-	lua_common_log_line(G_LOG_LEVEL_WARNING, L, msg, NULL, NULL, 1);
-	return 0;
+	return lua_logger_warnx(L);
 }
 
-static gint
+static int
 lua_logger_info(lua_State *L)
 {
-	LUA_TRACE_POINT;
-	const gchar *msg;
-	msg = luaL_checkstring(L, 1);
-	lua_common_log_line(G_LOG_LEVEL_INFO, L, msg, NULL, NULL, 1);
-	return 0;
+	return lua_logger_infox(L);
 }
 
-static gint
+static int
 lua_logger_message(lua_State *L)
 {
-	LUA_TRACE_POINT;
-	const gchar *msg;
-	msg = luaL_checkstring(L, 1);
-	lua_common_log_line(G_LOG_LEVEL_MESSAGE, L, msg, NULL, NULL, 1);
-	return 0;
+	return lua_logger_messagex(L);
 }
 
-static gint
+static int
 lua_logger_debug(lua_State *L)
 {
-	LUA_TRACE_POINT;
-	const gchar *msg;
-	msg = luaL_checkstring(L, 1);
-	lua_common_log_line(G_LOG_LEVEL_DEBUG, L, msg, NULL, NULL, 1);
-	return 0;
+	return lua_logger_debugx(L);
 }
 
 static inline bool
@@ -303,14 +280,14 @@ lua_logger_char_safe(int t, unsigned int esc_type)
 }
 
 static gsize
-lua_logger_out_str(lua_State *L, gint pos,
-				   gchar *outbuf, gsize len,
+lua_logger_out_str(lua_State *L, int pos,
+				   char *outbuf, gsize len,
 				   struct lua_logger_trace *trace,
 				   enum lua_logger_escape_type esc_type)
 {
 	gsize slen, flen;
-	const gchar *str = lua_tolstring(L, pos, &slen);
-	static const gchar hexdigests[16] = "0123456789abcdef";
+	const char *str = lua_tolstring(L, pos, &slen);
+	static const char hexdigests[16] = "0123456789abcdef";
 	gsize r = 0, s;
 
 	if (str) {
@@ -328,7 +305,7 @@ lua_logger_out_str(lua_State *L, gint pos,
 			r = rspamd_strlcpy(outbuf, str, flen + 1);
 		}
 		else {
-			/* Need to escape non printed characters */
+			/* Need to escape non-printed characters */
 			r = 0;
 			s = 0;
 
@@ -362,14 +339,14 @@ lua_logger_out_str(lua_State *L, gint pos,
 }
 
 static gsize
-lua_logger_out_num(lua_State *L, gint pos, gchar *outbuf, gsize len,
+lua_logger_out_num(lua_State *L, int pos, char *outbuf, gsize len,
 				   struct lua_logger_trace *trace)
 {
-	gdouble num = lua_tonumber(L, pos);
+	double num = lua_tonumber(L, pos);
 	glong inum;
 	gsize r = 0;
 
-	if ((gdouble) (glong) num == num) {
+	if ((double) (glong) num == num) {
 		inum = num;
 		r = rspamd_snprintf(outbuf, len + 1, "%l", inum);
 	}
@@ -381,7 +358,7 @@ lua_logger_out_num(lua_State *L, gint pos, gchar *outbuf, gsize len,
 }
 
 static gsize
-lua_logger_out_boolean(lua_State *L, gint pos, gchar *outbuf, gsize len,
+lua_logger_out_boolean(lua_State *L, int pos, char *outbuf, gsize len,
 					   struct lua_logger_trace *trace)
 {
 	gboolean val = lua_toboolean(L, pos);
@@ -393,11 +370,11 @@ lua_logger_out_boolean(lua_State *L, gint pos, gchar *outbuf, gsize len,
 }
 
 static gsize
-lua_logger_out_userdata(lua_State *L, gint pos, gchar *outbuf, gsize len,
+lua_logger_out_userdata(lua_State *L, int pos, char *outbuf, gsize len,
 						struct lua_logger_trace *trace)
 {
-	gint r = 0, top;
-	const gchar *str = NULL;
+	int r = 0, top;
+	const char *str = NULL;
 	gboolean converted_to_str = FALSE;
 
 	top = lua_gettop(L);
@@ -483,29 +460,30 @@ lua_logger_out_userdata(lua_State *L, gint pos, gchar *outbuf, gsize len,
 	return r;
 }
 
-#define MOVE_BUF(d, remain, r) \
-	(d) += (r);                \
-	(remain) -= (r);           \
-	if ((remain) == 0) {       \
-		lua_pop(L, 1);         \
-		break;                 \
+#define MOVE_BUF(d, remain, r)  \
+	(d) += (r);                 \
+	(remain) -= (r);            \
+	if ((remain) == 0) {        \
+		lua_settop(L, old_top); \
+		break;                  \
 	}
 
 static gsize
-lua_logger_out_table(lua_State *L, gint pos, gchar *outbuf, gsize len,
+lua_logger_out_table(lua_State *L, int pos, char *outbuf, gsize len,
 					 struct lua_logger_trace *trace,
 					 enum lua_logger_escape_type esc_type)
 {
-	gchar *d = outbuf;
+	char *d = outbuf;
 	gsize remain = len, r;
 	gboolean first = TRUE;
 	gconstpointer self = NULL;
-	gint i, tpos, last_seq = -1;
+	int i, tpos, last_seq = -1, old_top;
 
 	if (!lua_istable(L, pos) || remain == 0) {
 		return 0;
 	}
 
+	old_top = lua_gettop(L);
 	self = lua_topointer(L, pos);
 
 	/* Check if we have seen this pointer */
@@ -561,7 +539,6 @@ lua_logger_out_table(lua_State *L, gint pos, gchar *outbuf, gsize len,
 	/* Get string keys (pairs) */
 	for (lua_pushnil(L); lua_next(L, -2); lua_pop(L, 1)) {
 		/* 'key' is at index -2 and 'value' is at index -1 */
-
 		if (lua_type(L, -2) == LUA_TNUMBER) {
 			if (last_seq > 0) {
 				lua_pushvalue(L, -2);
@@ -600,7 +577,7 @@ lua_logger_out_table(lua_State *L, gint pos, gchar *outbuf, gsize len,
 		first = FALSE;
 	}
 
-	lua_pop(L, 1);
+	lua_settop(L, old_top);
 
 	r = rspamd_snprintf(d, remain + 1, "}");
 	d += r;
@@ -610,12 +587,12 @@ lua_logger_out_table(lua_State *L, gint pos, gchar *outbuf, gsize len,
 
 #undef MOVE_BUF
 
-gsize lua_logger_out_type(lua_State *L, gint pos,
-						  gchar *outbuf, gsize len,
+gsize lua_logger_out_type(lua_State *L, int pos,
+						  char *outbuf, gsize len,
 						  struct lua_logger_trace *trace,
 						  enum lua_logger_escape_type esc_type)
 {
-	gint type;
+	int type;
 	gsize r = 0;
 
 	if (len == 0) {
@@ -661,10 +638,10 @@ gsize lua_logger_out_type(lua_State *L, gint pos,
 	return r;
 }
 
-static const gchar *
-lua_logger_get_id(lua_State *L, gint pos, GError **err)
+static const char *
+lua_logger_get_id(lua_State *L, int pos, GError **err)
 {
-	const gchar *uid = NULL, *clsname;
+	const char *uid = NULL, *clsname;
 
 	if (lua_getmetatable(L, pos) != 0) {
 		uid = "";
@@ -676,7 +653,7 @@ lua_logger_get_id(lua_State *L, gint pos, GError **err)
 
 		clsname = lua_tostring(L, -1);
 
-		if (strcmp(clsname, "rspamd{task}") == 0) {
+		if (strcmp(clsname, rspamd_task_classname) == 0) {
 			struct rspamd_task *task = lua_check_task(L, pos);
 
 			if (task) {
@@ -687,7 +664,7 @@ lua_logger_get_id(lua_State *L, gint pos, GError **err)
 							EINVAL, "invalid rspamd{task}");
 			}
 		}
-		else if (strcmp(clsname, "rspamd{mempool}") == 0) {
+		else if (strcmp(clsname, rspamd_mempool_classname) == 0) {
 			rspamd_mempool_t *pool;
 
 			pool = rspamd_lua_check_mempool(L, pos);
@@ -700,7 +677,7 @@ lua_logger_get_id(lua_State *L, gint pos, GError **err)
 							EINVAL, "invalid rspamd{mempool}");
 			}
 		}
-		else if (strcmp(clsname, "rspamd{config}") == 0) {
+		else if (strcmp(clsname, rspamd_config_classname) == 0) {
 			struct rspamd_config *cfg;
 
 			cfg = lua_check_config(L, pos);
@@ -715,7 +692,7 @@ lua_logger_get_id(lua_State *L, gint pos, GError **err)
 							EINVAL, "invalid rspamd{config}");
 			}
 		}
-		else if (strcmp(clsname, "rspamd{map}") == 0) {
+		else if (strcmp(clsname, rspamd_map_classname) == 0) {
 			struct rspamd_lua_map *map;
 
 			map = lua_check_map(L, pos);
@@ -751,13 +728,13 @@ lua_logger_get_id(lua_State *L, gint pos, GError **err)
 }
 
 static gboolean
-lua_logger_log_format(lua_State *L, gint fmt_pos, gboolean is_string,
-					  gchar *logbuf, gsize remain)
+lua_logger_log_format(lua_State *L, int fmt_pos, gboolean is_string,
+					  char *logbuf, gsize remain)
 {
-	gchar *d;
-	const gchar *s, *c;
+	char *d;
+	const char *s, *c;
 	gsize r, cpylen = 0;
-	guint arg_num = 0, cur_arg;
+	unsigned int arg_num = 0, cur_arg;
 	bool num_arg = false;
 	struct lua_logger_trace tr;
 	enum {
@@ -822,7 +799,7 @@ lua_logger_log_format(lua_State *L, gint fmt_pos, gboolean is_string,
 					s++;
 				}
 
-				if (arg_num < 1 || arg_num > (guint) lua_gettop(L) + 1) {
+				if (arg_num < 1 || arg_num > (unsigned int) lua_gettop(L) + 1) {
 					msg_err("wrong argument number: %ud", arg_num);
 
 					return FALSE;
@@ -851,7 +828,7 @@ lua_logger_log_format(lua_State *L, gint fmt_pos, gboolean is_string,
 			arg_num = cur_arg;
 		}
 
-		if (arg_num < 1 || arg_num > (guint) lua_gettop(L) + 1) {
+		if (arg_num < 1 || arg_num > (unsigned int) lua_gettop(L) + 1) {
 			msg_err("wrong argument number: %ud", arg_num);
 
 			return FALSE;
@@ -877,16 +854,16 @@ lua_logger_log_format(lua_State *L, gint fmt_pos, gboolean is_string,
 	return TRUE;
 }
 
-static gint
+static int
 lua_logger_do_log(lua_State *L,
 				  GLogLevelFlags level,
 				  gboolean is_string,
-				  gint start_pos)
+				  int start_pos)
 {
-	gchar logbuf[RSPAMD_LOGBUF_SIZE - 128];
-	const gchar *uid = NULL;
-	gint fmt_pos = start_pos;
-	gint ret;
+	char logbuf[RSPAMD_LOGBUF_SIZE - 128];
+	const char *uid = NULL;
+	int fmt_pos = start_pos;
+	int ret;
 	GError *err = NULL;
 
 	if (lua_type(L, start_pos) == LUA_TSTRING) {
@@ -937,50 +914,50 @@ lua_logger_do_log(lua_State *L,
 	return 0;
 }
 
-static gint
+static int
 lua_logger_errx(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	return lua_logger_do_log(L, G_LOG_LEVEL_CRITICAL, FALSE, 1);
 }
 
-static gint
+static int
 lua_logger_warnx(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	return lua_logger_do_log(L, G_LOG_LEVEL_WARNING, FALSE, 1);
 }
 
-static gint
+static int
 lua_logger_infox(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	return lua_logger_do_log(L, G_LOG_LEVEL_INFO, FALSE, 1);
 }
 
-static gint
+static int
 lua_logger_messagex(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	return lua_logger_do_log(L, G_LOG_LEVEL_MESSAGE, FALSE, 1);
 }
 
-static gint
+static int
 lua_logger_debugx(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	return lua_logger_do_log(L, G_LOG_LEVEL_DEBUG, FALSE, 1);
 }
 
-static gint
+static int
 lua_logger_logx(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	GLogLevelFlags flags = lua_tonumber(L, 1);
-	const gchar *modname = lua_tostring(L, 2), *uid = NULL;
-	gchar logbuf[RSPAMD_LOGBUF_SIZE - 128];
+	const char *modname = lua_tostring(L, 2), *uid = NULL;
+	char logbuf[RSPAMD_LOGBUF_SIZE - 128];
 	gboolean ret;
-	gint stack_pos = 1;
+	int stack_pos = 1;
 
 	if (lua_type(L, 3) == LUA_TSTRING) {
 		uid = luaL_checkstring(L, 3);
@@ -1016,13 +993,13 @@ lua_logger_logx(lua_State *L)
 }
 
 
-static gint
+static int
 lua_logger_debugm(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gchar logbuf[RSPAMD_LOGBUF_SIZE - 128];
-	const gchar *uid = NULL, *module = NULL;
-	gint stack_pos = 1;
+	char logbuf[RSPAMD_LOGBUF_SIZE - 128];
+	const char *uid = NULL, *module = NULL;
+	int stack_pos = 1;
 	gboolean ret;
 
 	module = luaL_checkstring(L, 1);
@@ -1058,16 +1035,16 @@ lua_logger_debugm(lua_State *L)
 }
 
 
-static gint
+static int
 lua_logger_slog(lua_State *L)
 {
 	return lua_logger_do_log(L, 0, TRUE, 1);
 }
 
-static gint
+static int
 lua_logger_log_level(lua_State *L)
 {
-	gint log_level = rspamd_log_get_log_level(NULL);
+	int log_level = rspamd_log_get_log_level(NULL);
 
 	lua_pushstring(L, rspamd_get_log_severity_string(log_level));
 
@@ -1076,7 +1053,7 @@ lua_logger_log_level(lua_State *L)
 
 /*** Init functions ***/
 
-static gint
+static int
 lua_load_logger(lua_State *L)
 {
 	lua_newtable(L);
