@@ -32,7 +32,7 @@
 
 #include "unicode/uspoof.h"
 #include "unicode/uscript.h"
-#include "contrib/fastutf8/fastutf8.h"
+#include "rspamd_simdutf.h"
 
 /***
  * @module rspamd_util
@@ -644,9 +644,10 @@ LUA_FUNCTION_DEF(util, get_hostname);
 LUA_FUNCTION_DEF(util, parse_content_type);
 
 /***
- *  @function util.mime_header_encode(hdr)
+ *  @function util.mime_header_encode(hdr[, is_structured])
  * Encodes header if needed
  * @param {string} hdr input header
+ * @param {boolean} is_structured if true, then we encode as structured header (e.g. encode all non alpha-numeric characters)
  * @return encoded header
  */
 LUA_FUNCTION_DEF(util, mime_header_encode);
@@ -1025,7 +1026,12 @@ lua_util_encode_base64(lua_State *L)
 		}
 
 		if (out != NULL) {
-			lua_new_text(L, out, outlen, TRUE);
+			/*
+			 * Manually set OWN flag, as `lua_new_text` will allocate another chunk of memory,
+			 * and we will have memory leak of the memory allocated by `rspamd_encode_base64_fold`
+			 */
+			t = lua_new_text(L, out, outlen, FALSE);
+			t->flags = RSPAMD_TEXT_FLAG_OWN;
 		}
 		else {
 			lua_pushnil(L);
@@ -1650,7 +1656,9 @@ lua_util_transliterate(lua_State *L)
 
 	gsize outlen;
 	char *transliterated = rspamd_utf8_transliterate(t->start, t->len, &outlen);
-	lua_new_text(L, transliterated, outlen, TRUE);
+
+	t = lua_new_text(L, transliterated, outlen, FALSE);
+	t->flags = RSPAMD_TEXT_FLAG_OWN;
 
 	return 1;
 }
@@ -2144,6 +2152,17 @@ lua_util_is_utf_spoofed(lua_State *L)
 
 				return 1;
 			}
+
+			/* Disable single script confusables, as it is not what we want to check */
+			uspoof_setChecks(spc,
+							 USPOOF_CONFUSABLE & ~USPOOF_SINGLE_SCRIPT_CONFUSABLE,
+							 &uc_err);
+			if (uc_err != U_ZERO_ERROR) {
+				msg_err("Cannot set proper checks for uspoof: %s", u_errorName(uc_err));
+				lua_pushboolean(L, false);
+				uspoof_close(spc);
+				return 1;
+			}
 		}
 
 		ret = uspoof_areConfusableUTF8(spc, s1, l1, s2, l2, &uc_err);
@@ -2166,7 +2185,7 @@ lua_util_is_utf_spoofed(lua_State *L)
 			if (uc_err != U_ZERO_ERROR) {
 				msg_err("Cannot set proper checks for uspoof: %s", u_errorName(uc_err));
 				lua_pushboolean(L, false);
-				uspoof_close(spc);
+				uspoof_close(spc_sgl);
 				return 1;
 			}
 		}
@@ -2399,15 +2418,19 @@ static int
 lua_util_mime_header_encode(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gsize len;
-	const char *hdr = luaL_checklstring(L, 1, &len);
+	struct rspamd_lua_text *hdr = lua_check_text_or_string(L, 1);
 	char *encoded;
+	bool is_structured = false;
 
 	if (!hdr) {
 		return luaL_error(L, "invalid arguments");
 	}
 
-	encoded = rspamd_mime_header_encode(hdr, len);
+	if (lua_isboolean(L, 2)) {
+		is_structured = lua_toboolean(L, 2);
+	}
+
+	encoded = rspamd_mime_header_encode(hdr->start, hdr->len, is_structured);
 	lua_pushstring(L, encoded);
 	g_free(encoded);
 
